==> vmaid-2.0.9.tar.gz: vmaid-2.0.9/src/sigmain.c <==
/*
    Video maid
    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "command.h"
#include "file.h"
#include "general.h"
#include "sigfile.h"
#include "sigmain.h"
#include "thread.h"
#include "misc/argument.h"
#include "orz/orzhistory.h"
#include "orz/orzmdi.h"


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(テキスト)                                        *
*                                                                             *
******************************************************************************/
void
signal_style_set (GtkWidget *widget,
                  GtkStyle  *style,
                  gpointer   user_data)
{
  gboolean success[4];
  gint i, ascent, height;

  style = gtk_widget_get_style (widget);
  /* ja:システム色 */
  system_color[0] = style->text[0];
  system_color[1] = style->base[0];
  system_color[2] = style->dark[0];
  system_color[3] = style->bg[3];
  gdk_colormap_alloc_colors (gdk_colormap_get_system (),
                                        system_color, 4, FALSE, TRUE, success);
  if (system_font)
    gdk_font_unref (system_font);
  /* ja:システムフォント */
  system_font = gdk_font_ref (gtk_style_get_font (style));
  for (i = 0; i < 256; i++)
    {
      gdk_text_extents (system_font, (gchar *)&i, 1,
                                            NULL, NULL, NULL, &ascent, NULL);
      if (system_font_ascent < ascent)
        system_font_ascent = ascent;
      height = gdk_char_height (system_font, (gchar)i);
      if (system_font_height < height)
        system_font_height = height;
    }
  for (i = orz_mdi_get_n_pages (ORZ_MDI (mdi)) - 1; i >= 0; i--)
    set_usize (orz_mdi_get_data (ORZ_MDI (mdi), i));
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(メニュー)                                        *
*                                                                             *
******************************************************************************/
void
signal_activate_menu_history (GtkWidget   *widget,
                              const gchar *file)
{
  FileOpen *file_open;

  file_open = g_malloc (sizeof (FileOpen));
  file_open->file = g_strdup (file);
  file_open->merge = FALSE;
#ifdef USE_THREAD
  g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
#else /* not USE_THREAD */
  file_open_edit (file_open);
#endif /* not USE_THREAD */
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(MDI)                                             *
*                                                                             *
******************************************************************************/
void
signal_close_window (GtkWidget   *widget,
                     guint        page_num,
                     VmaidWindow *vmaid)
{
  if (prompt_close (vmaid))
    gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page_num);
}


void
signal_switch_window (GtkWidget   *widget,
                      guint        page_num,
                      VmaidWindow *vmaid)
{
  /* ja:メニュー表示 */
  set_menu_bar (vmaid);
}


void
signal_destroy_mdi (GtkWidget *widget,
                    gpointer   user_data)
{
  gtk_timeout_remove (timer_id);
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(リスト)                                          *
*                                                                             *
******************************************************************************/
static void
signal_tree_selection_foreach (GtkTreeModel *model,
                               GtkTreePath  *path,
                               GtkTreeIter  *iter,
                               gpointer      user_data)
{
  (*(gint *)user_data)++;
}


void
signal_changed (GtkTreeSelection *select,
                gpointer          user_data)
{
  gint count = 0;

  gtk_tree_selection_selected_foreach (select, signal_tree_selection_foreach,
                                                                    &count);
  gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/task"), count > 0);
}


gboolean
signal_button_press_tview (GtkWidget        *widget,
                           GdkEventButton   *event,
                           GtkTreeSelection *select)
{
  if (event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
      gint count = 0;
      GtkWidget *menu_shell;

      gtk_tree_selection_selected_foreach (select,
                                        signal_tree_selection_foreach, &count);
      gtk_widget_set_sensitive (misc_find_menu (tview_entries, "/cancel"),
                                                                    count > 0);
      menu_shell = misc_find_menu (tview_entries, "/");
      gtk_widget_show_all (menu_shell);
      gtk_menu_popup (GTK_MENU (menu_shell),
                        NULL, NULL, NULL, NULL, event->button, event->time);
      return TRUE;
    }
  return FALSE;
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(セレクション)                                    *
*                                                                             *
******************************************************************************/
void
signal_drag_data_received (GtkWidget        *widget,
                           GdkDragContext   *context,
                           gint              x,
                           gint              y,
                           GtkSelectionData *selection_data,
                           guint             info,
                           guint             time)
{
  gchar **files;
  gint i;

  files = g_strsplit ((const gchar *)selection_data->data, "\r\n", 0);
  for (i = 0; files[i]; i++)
    {
      gchar *file;

      file = g_filename_from_uri (files[i], NULL, NULL);
      if (file)
        {
          FileOpen *file_open;

          file_open = g_malloc (sizeof (FileOpen));
          file_open->file = file;
          file_open->merge = FALSE;
#ifdef USE_THREAD
          g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
#else /* not USE_THREAD */
          file_open_edit (file_open);
#endif /* not USE_THREAD */
        }
    }
  g_strfreev (files);
}


void
signal_selection_paste (GObject   *object,
                        AviEdit  **avi_edit,
                        gpointer   user_data)
{
  gboolean result = TRUE, paste[2];
  gint i, max, sx, page_num;
  VmaidWindow *vmaid;
  VmaidHistory *d;

  /* ja:貼り付け */
  if (!avi_edit)
    return;
  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
  vmaid = orz_mdi_get_data (ORZ_MDI (mdi), page_num);
  if (vmaid->select.stream < 0)
    {
      /* ja:選択範囲がないときUNDOの準備 */
      d = g_malloc0 (sizeof (VmaidHistory));
      for (i = 0; i < 2; i++)
        if (vmaid->avi_edit[i]
                    && !(d->avi_edit[i] = avi_edit_dup (vmaid->avi_edit[i])))
          result = FALSE;
      if (!result)
        {
          GtkWidget *dialog;

          dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                            _("AVI Clone Error"));
          g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                    G_CALLBACK (misc_dialog_key_press), NULL);
          gtk_dialog_run (GTK_DIALOG (dialog));
          gtk_widget_destroy (dialog);
          for (i = 0; i < 2; i++)
            if (d->avi_edit[i])
              avi_edit_close (d->avi_edit[i]);
          g_free (d);
          return;
        }
      d->cursor = vmaid->cursor;
      d->select = vmaid->select;
      d->next = vmaid->undo;
      vmaid->undo = d;
    }
  else
    {
      /* ja:選択範囲があればそれを削除する */
      d = vmaid->redo;
      vmaid->redo = NULL;
      command_delete (window, NULL);
      vmaid->redo = d;
    }
  paste[0] = paste[1] = FALSE;
  for (i = 0; avi_edit[i]; i++)
    switch (avi_edit_type (avi_edit[i]))
      {
        case AVI_TYPE_VIDEO:
          if (!paste[0])
            {
              if (!vmaid->avi_edit[0])
                {
                  vmaid->avi_edit[0] = avi_edit_dup (avi_edit[i]);
                }
              else if (!avi_edit_paste (vmaid->avi_edit[0],
                                        MIN (vmaid->cursor.frame,
                                        avi_edit_length (vmaid->avi_edit[0])),
                                                avi_edit_dup (avi_edit[i])))
                result = FALSE;
              paste[0] = TRUE;
            }
          break;
        case AVI_TYPE_AUDIO:
          if (!paste[1])
            {
              if (!vmaid->avi_edit[1])
                {
                  vmaid->avi_edit[1] = avi_edit_dup (avi_edit[i]);
                }
              else if (!avi_edit_paste (vmaid->avi_edit[1],
                            MIN (avi_time_to_sample (vmaid->avi_edit[1],
                                (gint)((glonglong)vmaid->cursor.frame * 1000
                                                * vmaid->scale / vmaid->rate)),
                                        avi_edit_length (vmaid->avi_edit[1])),
                                                avi_edit_dup (avi_edit[i])))
                result = FALSE;
              paste[1] = TRUE;
            }
          break;
        default:
          result = FALSE;
        }
  if (!result || (!paste[0] && !paste[1]))
    {
      GtkWidget *dialog;

      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                            _("AVI Paste Error"));
      g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                    G_CALLBACK (misc_dialog_key_press), NULL);
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      for (i = 0; i < 2; i++)
        if (vmaid->avi_edit[i])
          avi_edit_close (vmaid->avi_edit[i]);
      d = vmaid->undo;
      vmaid->undo = vmaid->undo->next;
      for (i = 0; i < 2; i++)
        vmaid->avi_edit[i] = d->avi_edit[i];
      g_free (d);
      return;
    }
  if (vmaid->avi_edit[0])
    {
      vmaid->rate = avi_edit_get_rate (vmaid->avi_edit[0]);
      vmaid->scale = avi_edit_get_scale (vmaid->avi_edit[0]);
    }
  max = get_max_frame (vmaid, -1) + 1;
  sx = get_sx_frame (vmaid);
  if (vmaid->avi_edit[0] && !vmaid->avi_edit[1])
    vmaid->cursor.stream = 0;
  else if (!vmaid->avi_edit[0] && vmaid->avi_edit[1])
    vmaid->cursor.stream = 1;
  if (vmaid->cursor.frame > max)
    vmaid->cursor.frame = max;
  /* ja:キャレットが表示されるように調整 */
  if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
    vmaid->top = MIN (vmaid->cursor.frame, max - sx);
  if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
    vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
  /* ja:メニュー,ステートメント,子ウインドウ */
  set_menu_bar (vmaid);
  set_usize (vmaid);
  draw_caret (vmaid, NULL);
  misc_set_scroll_bar (vmaid->hscroll,
                                GTK_SIGNAL_FUNC (signal_value_changed_hscroll),
                                                vmaid, 0, max, sx, vmaid->top);
  orz_mdi_set_edited (ORZ_MDI (mdi), page_num, TRUE);
}


void
signal_selection_recv (GObject   *object,
                       gchar    **argv,
                       gpointer   user_data)
{
  gboolean *arg_files;
  gint i, array[2], def[2];
  ArgumentBool arg_n, arg_x;
  ArgumentValue arg_h;
  ArgumentArray arg_s;
  ArgumentList arg_opts[] = {
{"history", 'h',  "NUM",          N_("History"),     ARGUMENT_TYPE_VALUE, &arg_h},
{"new",     'n',  "ON/OFF",       N_("New File"),    ARGUMENT_TYPE_BOOL,  &arg_n},
{"size",    's',  "WIDTH,HEIGHT", N_("Window Size"), ARGUMENT_TYPE_ARRAY, &arg_s},
{"second",  'x',  "ON/OFF",       N_("Open Window"), ARGUMENT_TYPE_BOOL,  &arg_x},
{NULL,      '\0', NULL,           NULL,              0,                   NULL}};

  /* ja:プロセス間通信 */
  /* ja:ファイルの履歴 */
  arg_h.val = orz_history_get_num (ORZ_HISTORY (history));
  arg_h.def = ORZ_HISTORY_NUM_DEF;
  /* ja:新規ファイル */
  arg_n.flag = newfile;
  arg_n.def = FALSE;
  /* ja:ウインドウサイズ */
  array[0] = def_width;
  array[1] = def_height;
  def[0] = gdk_screen_width () * 2 / 3;
  def[1] = gdk_screen_height () * 2 / 3;
  arg_s.leng = 2;
  arg_s.array = array;
  arg_s.def = def;
  /* ja:新規ウインドウ */
  arg_x.flag = second;
  arg_x.def = FALSE;
  /* ja:コマンドラインの解析 */
  arg_files = arg_analyse (argv, arg_opts);
  orz_history_set_num (ORZ_HISTORY (history), arg_h.val);
  newfile = arg_n.flag;
  def_width = array[0];
  def_height = array[1];
  second = arg_x.flag;
  for (i = 1; argv[i]; i++)
    if (arg_files[i])
      {
        FileOpen *file_open;

        file_open = g_malloc (sizeof (FileOpen));
        file_open->file = g_strdup (argv[i]);
        file_open->merge = FALSE;
#ifdef USE_THREAD
        g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
#else /* not USE_THREAD */
        file_open_edit (file_open);
#endif /* not USE_THREAD */
      }
  g_free (arg_files);
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(メイン)                                          *
*                                                                             *
******************************************************************************/
gboolean
signal_delete (GtkWidget *widget,
               GdkEvent  *event,
               gpointer   user_data)
{
  gint page;

  while ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0)
    {
      if (!prompt_close ((VmaidWindow *)
                                    orz_mdi_get_data (ORZ_MDI (mdi), page)))
        return TRUE;
      gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page);
    }
#ifdef USE_THREAD
  g_thread_create ((GThreadFunc)close_program, NULL, TRUE, NULL);
#else /* not USE_THREAD */
  close_program ();
#endif /* not USE_THREAD */
  return TRUE;
}

==> vmaid-2.1.2.tar.gz: vmaid-2.1.2/src/sigmain.c <==
/*
    Video maid
    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "command.h"
#include "file.h"
#include "general.h"
#include "sigfile.h"
#include "sigmain.h"
#include "thread.h"
#include "misc/argument.h"
#include "orz/orzhistory.h"
#include "orz/orzmdi.h"


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(テキスト)                                        *
*                                                                             *
******************************************************************************/
void
signal_style_set (GtkWidget *widget,
                  GtkStyle  *style,
                  gpointer   user_data)
{
  gboolean success[4];
  gint i;

  style = gtk_widget_get_style (widget);
  /* ja:システム色 */
  system_color[0] = style->text[0];
  system_color[1] = style->base[0];
  system_color[2] = style->dark[0];
  system_color[3] = style->bg[3];
  gdk_colormap_alloc_colors (gdk_colormap_get_system (),
                                        system_color, 4, FALSE, TRUE, success);
  /* ja:システムフォント */
  if (system_font_layout)
    g_object_unref (system_font_layout);
  system_font_layout = gtk_widget_create_pango_layout (widget, NULL);
  for (i = 0; i < 128; i++)
    if (g_unichar_isprint ((gunichar)i))
      {
        int w, h;

        pango_layout_set_text (system_font_layout, (const char *)&i, 1);
        pango_layout_get_pixel_size (system_font_layout, &w, &h);
        if (system_font_height < h)
          system_font_height = h;
      }
  for (i = orz_mdi_get_n_pages (ORZ_MDI (mdi)) - 1; i >= 0; i--)
    set_usize (orz_mdi_get_data (ORZ_MDI (mdi), i));
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(メニュー)                                        *
*                                                                             *
******************************************************************************/
void
signal_activate_menu_history (GtkWidget   *widget,
                              const gchar *file)
{
  FileOpen *file_open;

  file_open = g_malloc (sizeof (FileOpen));
  file_open->file = g_strdup (file);
  file_open->merge = FALSE;
#ifdef USE_THREAD
  g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
#else /* not USE_THREAD */
  file_open_edit (file_open);
#endif /* not USE_THREAD */
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(MDI)                                             *
*                                                                             *
******************************************************************************/
void
signal_close_window (GtkWidget   *widget,
                     guint        page_num,
                     VmaidWindow *vmaid)
{
  if (prompt_close (vmaid))
    gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page_num);
}


void
signal_switch_window (GtkWidget   *widget,
                      guint        page_num,
                      VmaidWindow *vmaid)
{
  /* ja:メニュー表示 */
  set_menu_bar (vmaid);
}


void
signal_destroy_mdi (GtkWidget *widget,
                    gpointer   user_data)
{
  g_source_remove (timer_id);
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(リスト)                                          *
*                                                                             *
******************************************************************************/
static void
signal_tree_selection_foreach (GtkTreeModel *model,
                               GtkTreePath  *path,
                               GtkTreeIter  *iter,
                               gpointer      user_data)
{
  (*(gint *)user_data)++;
}


void
signal_changed (GtkTreeSelection *select,
                gpointer          user_data)
{
  gint count = 0;

  gtk_tree_selection_selected_foreach (select, signal_tree_selection_foreach,
                                                                    &count);
  gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/task"), count > 0);
}


gboolean
signal_button_press_tview (GtkWidget        *widget,
                           GdkEventButton   *event,
                           GtkTreeSelection *select)
{
  if (event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
      gint count = 0;
      GtkWidget *menu_shell;

      gtk_tree_selection_selected_foreach (select,
                                        signal_tree_selection_foreach, &count);
      gtk_widget_set_sensitive (misc_find_menu (tview_entries, "/cancel"),
                                                                    count > 0);
      menu_shell = misc_find_menu (tview_entries, "/");
      gtk_widget_show_all (menu_shell);
      gtk_menu_popup (GTK_MENU (menu_shell),
                        NULL, NULL, NULL, NULL, event->button, event->time);
      return TRUE;
    }
  return FALSE;
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(セレクション)                                    *
*                                                                             *
******************************************************************************/
void
signal_drag_data_received (GtkWidget        *widget,
                           GdkDragContext   *context,
                           gint              x,
                           gint              y,
                           GtkSelectionData *selection_data,
                           guint             info,
                           guint             time)
{
  gchar **files;
  gint i;

  files = g_strsplit ((const gchar *)selection_data->data, "\r\n", 0);
  for (i = 0; files[i]; i++)
    {
      gchar *file;

      file = g_filename_from_uri (files[i], NULL, NULL);
      if (file)
        {
          FileOpen *file_open;

          file_open = g_malloc (sizeof (FileOpen));
          file_open->file = file;
          file_open->merge = FALSE;
#ifdef USE_THREAD
          g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
#else /* not USE_THREAD */
          file_open_edit (file_open);
#endif /* not USE_THREAD */
        }
    }
  g_strfreev (files);
}


void
signal_selection_paste (GObject   *object,
                        AviEdit  **avi_edit,
                        gpointer   user_data)
{
  gboolean result = TRUE, paste[2];
  gint i, max, sx, page_num;
  VmaidWindow *vmaid;
  VmaidHistory *d;

  /* ja:貼り付け */
  if (!avi_edit)
    return;
  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
  vmaid = orz_mdi_get_data (ORZ_MDI (mdi), page_num);
  if (vmaid->select.stream < 0)
    {
      /* ja:選択範囲がないときUNDOの準備 */
      d = g_malloc0 (sizeof (VmaidHistory));
      for (i = 0; i < 2; i++)
        if (vmaid->avi_edit[i]
                    && !(d->avi_edit[i] = avi_edit_dup (vmaid->avi_edit[i])))
          result = FALSE;
      if (!result)
        {
          GtkWidget *dialog;

          dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                            _("AVI Clone Error"));
          g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                    G_CALLBACK (misc_dialog_key_press), NULL);
          gtk_dialog_run (GTK_DIALOG (dialog));
          gtk_widget_destroy (dialog);
          for (i = 0; i < 2; i++)
            if (d->avi_edit[i])
              avi_edit_close (d->avi_edit[i]);
          g_free (d);
          return;
        }
      d->cursor = vmaid->cursor;
      d->select = vmaid->select;
      d->next = vmaid->undo;
      vmaid->undo = d;
    }
  else
    {
      /* ja:選択範囲があればそれを削除する */
      d = vmaid->redo;
      vmaid->redo = NULL;
      command_delete (window, NULL);
      vmaid->redo = d;
    }
  paste[0] = paste[1] = FALSE;
  for (i = 0; avi_edit[i]; i++)
    switch (avi_edit_type (avi_edit[i]))
      {
        case AVI_TYPE_VIDEO:
          if (!paste[0])
            {
              if (!vmaid->avi_edit[0])
                {
                  vmaid->avi_edit[0] = avi_edit_dup (avi_edit[i]);
                }
              else if (!avi_edit_paste (vmaid->avi_edit[0],
                                        MIN (vmaid->cursor.frame,
                                        avi_edit_length (vmaid->avi_edit[0])),
                                                avi_edit_dup (avi_edit[i])))
                result = FALSE;
              paste[0] = TRUE;
            }
          break;
        case AVI_TYPE_AUDIO:
          if (!paste[1])
            {
              if (!vmaid->avi_edit[1])
                {
                  vmaid->avi_edit[1] = avi_edit_dup (avi_edit[i]);
                }
              else if (!avi_edit_paste (vmaid->avi_edit[1],
                            MIN (avi_time_to_sample (vmaid->avi_edit[1],
                                (gint)((glonglong)vmaid->cursor.frame * 1000
                                                * vmaid->scale / vmaid->rate)),
                                        avi_edit_length (vmaid->avi_edit[1])),
                                                avi_edit_dup (avi_edit[i])))
                result = FALSE;
              paste[1] = TRUE;
            }
          break;
        default:
          result = FALSE;
        }
  if (!result || (!paste[0] && !paste[1]))
    {
      GtkWidget *dialog;

      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
                            _("AVI Paste Error"));
      g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                    G_CALLBACK (misc_dialog_key_press), NULL);
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      for (i = 0; i < 2; i++)
        if (vmaid->avi_edit[i])
          avi_edit_close (vmaid->avi_edit[i]);
      d = vmaid->undo;
      vmaid->undo = vmaid->undo->next;
      for (i = 0; i < 2; i++)
        vmaid->avi_edit[i] = d->avi_edit[i];
      g_free (d);
      return;
    }
  if (vmaid->avi_edit[0])
    {
      vmaid->rate = avi_edit_get_rate (vmaid->avi_edit[0]);
      vmaid->scale = avi_edit_get_scale (vmaid->avi_edit[0]);
    }
  max = get_max_frame (vmaid, -1) + 1;
  sx = get_sx_frame (vmaid);
  if (vmaid->avi_edit[0] && !vmaid->avi_edit[1])
    vmaid->cursor.stream = 0;
  else if (!vmaid->avi_edit[0] && vmaid->avi_edit[1])
    vmaid->cursor.stream = 1;
  if (vmaid->cursor.frame > max)
    vmaid->cursor.frame = max;
  /* ja:キャレットが表示されるように調整 */
  if (vmaid->top > MIN (vmaid->cursor.frame, max - sx))
    vmaid->top = MIN (vmaid->cursor.frame, max - sx);
  if (vmaid->top < MAX (vmaid->cursor.frame - sx, 0))
    vmaid->top = MAX (vmaid->cursor.frame - sx, 0);
  /* ja:メニュー,ステートメント,子ウインドウ */
  set_menu_bar (vmaid);
  set_usize (vmaid);
  draw_caret (vmaid, NULL);
  misc_set_scroll_bar (vmaid->hscroll,
                                GTK_SIGNAL_FUNC (signal_value_changed_hscroll),
                                                vmaid, 0, max, sx, vmaid->top);
  orz_mdi_set_edited (ORZ_MDI (mdi), page_num, TRUE);
}


void
signal_selection_recv (GObject   *object,
                       gchar    **argv,
                       gpointer   user_data)
{
  gboolean *arg_files;
  gint i, array[2], def[2];
  ArgumentBool arg_n, arg_x;
  ArgumentValue arg_h;
  ArgumentArray arg_s;
  ArgumentList arg_opts[] = {
{"history", 'h',  "NUM",          N_("History"),     ARGUMENT_TYPE_VALUE, &arg_h},
{"new",     'n',  "ON/OFF",       N_("New File"),    ARGUMENT_TYPE_BOOL,  &arg_n},
{"size",    's',  "WIDTH,HEIGHT", N_("Window Size"), ARGUMENT_TYPE_ARRAY, &arg_s},
{"second",  'x',  "ON/OFF",       N_("Open Window"), ARGUMENT_TYPE_BOOL,  &arg_x},
{NULL,      '\0', NULL,           NULL,              0,                   NULL}};

  /* ja:プロセス間通信 */
  /* ja:ファイルの履歴 */
  arg_h.val = orz_history_get_num (ORZ_HISTORY (history));
  arg_h.def = ORZ_HISTORY_NUM_DEF;
  /* ja:新規ファイル */
  arg_n.flag = newfile;
  arg_n.def = FALSE;
  /* ja:ウインドウサイズ */
  array[0] = def_width;
  array[1] = def_height;
  def[0] = gdk_screen_width () * 2 / 3;
  def[1] = gdk_screen_height () * 2 / 3;
  arg_s.leng = 2;
  arg_s.array = array;
  arg_s.def = def;
  /* ja:新規ウインドウ */
  arg_x.flag = second;
  arg_x.def = FALSE;
  /* ja:コマンドラインの解析 */
  arg_files = arg_analyse (argv, arg_opts);
  /* ja:ファイルの履歴 */
  orz_history_set_num (ORZ_HISTORY (history), arg_h.val);
  /* ja:新規ファイル */
  newfile = arg_n.flag;
  /* ja:ウインドウサイズ */
  def_width = array[0];
  def_height = array[1];
  /* ja:新規ウインドウ */
  second = arg_x.flag;
  for (i = 1; argv[i]; i++)
    if (arg_files[i])
      {
        FileOpen *file_open;

        file_open = g_malloc (sizeof (FileOpen));
        file_open->file = g_strdup (argv[i]);
        file_open->merge = FALSE;
#ifdef USE_THREAD
        g_thread_create ((GThreadFunc)file_open_edit, file_open, TRUE, NULL);
#else /* not USE_THREAD */
        file_open_edit (file_open);
#endif /* not USE_THREAD */
      }
  g_free (arg_files);
}


/******************************************************************************
*                                                                             *
* ja:シグナル/イベント関数群(メイン)                                          *
*                                                                             *
******************************************************************************/
gboolean
signal_delete (GtkWidget *widget,
               GdkEvent  *event,
               gpointer   user_data)
{
  gint page;

  while ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0)
    {
      if (!prompt_close ((VmaidWindow *)
                                    orz_mdi_get_data (ORZ_MDI (mdi), page)))
        return TRUE;
      gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page);
    }
#ifdef USE_THREAD
  g_thread_create ((GThreadFunc)close_program, NULL, TRUE, NULL);
#else /* not USE_THREAD */
  close_program ();
#endif /* not USE_THREAD */
  return TRUE;
}
